/*
 *  Galaxian Clone Test -- WiFiBoy32 Demo Code for Arduino/ESP32
 *
 *  Feb 18, 2017 Created. (derek@wifiboy.org)
 *  Oct 18, 2017 Library Updated. (tklua@wifiboy.org)
 *
 */
#include <wifiboy32.h>
#include <math.h>
#include "wb32-spritemap256c.h"

uint8_t m_x[100];
uint16_t m_y[100];
uint8_t m_v[100];
uint16_t m_c[100];
int xx=60, yy, shooting=0;
uint8_t wb32_buf8[76800]; 

void blit_str256(const char *str, int x, int y)
{
    for(int i=0; i<strlen(str); i++) {
        if (str[i]>='@'&& str[i]<=']') wb32_blitBuf8(8*(str[i]-'@'),0,240, x+i*8, y, 8, 8, (uint8_t *)sprites);
        if (str[i]>='!'&& str[i]<='>') wb32_blitBuf8(8*(str[i]-'!'),8,240, x+i*8, y, 8, 8, (uint8_t *)sprites);
        if (str[i]=='?') wb32_blitBuf8(8*14,16,240, x+i*8, y, 8, 8, (uint8_t *)sprites);
        if (str[i]=='c') wb32_blitBuf8(8*13,16,240, x+i*8, y, 8, 8, (uint8_t *)sprites);
        if (str[i]=='w') wb32_blitBuf8(7,16,240, x+i*8, y, 26, 8, (uint8_t *)sprites);
        if (str[i]=='x') wb32_blitBuf8(42,16,240, x+i*8, y, 61, 8, (uint8_t *)sprites);
    }
}

void blit_num256(uint16_t num, uint16_t x, uint16_t y, uint8_t color_mode)
{
    uint16_t  d[5];
    
    d[0]=num/10000;
    d[1]=(num-d[0]*10000)/1000;
    d[2]=(num-d[0]*10000-d[1]*1000)/100;
    d[3]=(num-d[0]*10000-d[1]*1000-d[2]*100)/10;
    d[4]=num-d[0]*10000-d[1]*1000-d[2]*100-d[3]/10;
    
    for(int i=0; i<5; i++) {
        wb32_blitBuf8(d[i]*8+120, color_mode*8, 240, x+i*8, y, 8, 8, (uint8_t *)sprites);
    }
}


void setup() {
  wb32_init();
  wb32_fillScreen(wbBLACK);
  
  for(int i=0; i<100; i++) {
    m_x[i]=random(0,240);
    m_y[i]=random(0,320);
    m_v[i]=random(2,5);
    m_c[i]=random(0,16);
  }
  pinMode(17,INPUT);
  pinMode(32,INPUT);
  pinMode(34,INPUT);
  pinMode(35,INPUT);
  
  for(int i=0; i<256; i++) wb32_setPal8(i, wb32_color565(sprite_pal[i][0],sprite_pal[i][1],sprite_pal[i][2]));
}

void loop() {

    // getkey
    if (digitalRead(17)==0) if (xx>1) xx-=3;
    if (digitalRead(32)==0) if (xx<222) xx+=3;
    if (digitalRead(34)==0) {
        if (shooting==0) {
            yy=275;
            shooting=1;
        }
    }

    wb32_clearBuf8();

   // stars
    for(int i=0; i<100; i++) {
        wb32_setBuf8(m_x[i]+m_y[i]*240, m_c[i]);
        m_y[i] += m_v[i];
        if (m_y[i]>=320) m_y[i]-=320;
    }

    // bullet
    if (shooting==1) {
        yy-=5; if (yy<0) { yy=275; shooting=0;}
        wb32_blitBuf8(11, 70, 240, xx+7,yy, 2,5,(uint8_t *)sprites);
    } else wb32_blitBuf8(11, 70, 240, xx+7,275, 2,5,(uint8_t *)sprites);

    // fighter
    wb32_blitBuf8(4, 75, 240, xx, 280, 16, 21, (uint8_t *)sprites);
    
    // aliens
    wb32_blitBuf8(3, 150, 240, 17+72, 50, 11, 11, (uint8_t *)sprites);
    wb32_blitBuf8(3, 150, 240, 4*17+72, 50, 11, 11, (uint8_t *)sprites);
    for(int y=1; y<2; y++)
        for(int i=0; i<6; i++) wb32_blitBuf8(2, 103, 240, i*17+72, 50+y*13, 11, 8, (uint8_t *)sprites);
    for(int y=2; y<3; y++)
        for(int i=0; i<8; i++) wb32_blitBuf8(2, 119, 240, i*17+55, 50+y*13, 11, 8, (uint8_t *)sprites);
    for(int y=3; y<6; y++)
        for(int i=0; i<10; i++) wb32_blitBuf8(2, 135, 240, i*17+38, 50+y*13, 11, 8, (uint8_t *)sprites);
        
    for(int i=0; i<2; i++) wb32_blitBuf8(56, 84, 240, i*10+220, 305, 8, 13, (uint8_t *)sprites); // flag
    for(int i=0; i<4; i++) wb32_blitBuf8(42, 82, 240, i*15, 305, 11, 14, (uint8_t *)sprites); // lives

    // text
    blit_str256("GALAXIAN CLONE TEST", 45,160);
    blit_str256("HIGH SCORE", 80,0);
    blit_num256(1800, 20, 10, 2);
    blit_num256(12800, 100, 10, 2);

    blit_str256("c2017 x", 64,200);

    // show up
    wb32_blit8();
}
